var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { nth } from 'lodash';
import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import { useEventCallback, useId, useIsomorphicLayoutEffect, useResize } from '@react-devui/hooks';
import { CloseOutlined, EllipsisOutlined, PlusOutlined } from '@react-devui/icons';
import { checkScrollEnd, getClassName } from '@react-devui/utils';
import { useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DDropdown } from '../dropdown';
import { useComponentConfig, usePrefixConfig, useTranslation } from '../root';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTabs' });
function Tabs(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dList, dActive, dPlacement = 'top', dCenter = false, dType, dSize, onActiveChange, onAddClick, onClose } = _a, restProps = __rest(_a, ["dList", "dActive", "dPlacement", "dCenter", "dType", "dSize", "onActiveChange", "onAddClick", "onClose"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const tabsRef = useRef(null);
    const tablistWrapperRef = useRef(null);
    const tablistRef = useRef(null);
    const indicatorRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const uniqueId = useId();
    const getTabId = (id) => `${dPrefix}tabs-tab-${id}-${uniqueId}`;
    const getPanelId = (id) => `${dPrefix}tabs-panel-${id}-${uniqueId}`;
    const [listOverflow, setListOverflow] = useState(false);
    const [dropdownList, setDropdownList] = useState([]);
    const [scrollEnd, setScrollEnd] = useState(false);
    const iconSize = dSize === 'smaller' ? 16 : dSize === 'larger' ? 20 : 18;
    const isHorizontal = dPlacement === 'top' || dPlacement === 'bottom';
    const [activeId, changeActiveId] = useDValue(() => {
        for (const tab of dList) {
            if (!tab.disabled) {
                return tab.id;
            }
        }
    }, dActive, (id) => {
        if (onActiveChange) {
            const tab = dList.find((t) => t.id === id);
            if (tab) {
                onActiveChange(id, tab);
            }
        }
    });
    const refreshTabs = () => {
        const tablistWrapperEl = tablistWrapperRef.current;
        if (tablistWrapperEl) {
            const isOverflow = isHorizontal
                ? tablistWrapperEl.scrollWidth > tablistWrapperEl.clientWidth
                : tablistWrapperEl.scrollHeight > tablistWrapperEl.clientHeight;
            setListOverflow(isOverflow);
            setScrollEnd(checkScrollEnd(tablistWrapperEl)[isHorizontal ? 'x' : 'y']);
            if (isOverflow) {
                const tablistWrapperRect = tablistWrapperEl.getBoundingClientRect();
                const dropdownList = [];
                dList.forEach((tab) => {
                    const el = document.getElementById(getTabId(tab.id));
                    if (el) {
                        const rect = el.getBoundingClientRect();
                        if (isHorizontal) {
                            if (rect.right + 52 + (onAddClick ? 52 : 0) > tablistWrapperRect.right || rect.left < tablistWrapperRect.left) {
                                dropdownList.push(tab);
                            }
                        }
                        else {
                            if (rect.bottom + 36 + (onAddClick ? 36 : 0) > tablistWrapperRect.bottom || rect.top < tablistWrapperRect.top) {
                                dropdownList.push(tab);
                            }
                        }
                    }
                });
                setDropdownList(dropdownList);
            }
        }
    };
    useIsomorphicLayoutEffect(() => {
        refreshTabs();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    useResize(tablistWrapperRef, refreshTabs);
    useResize(tablistRef, refreshTabs);
    const updateIndicator = useEventCallback(() => {
        if (tablistRef.current && indicatorRef.current) {
            const tablistRect = tablistRef.current.getBoundingClientRect();
            for (const tab of dList) {
                if (tab.id === activeId) {
                    const el = document.getElementById(getTabId(tab.id));
                    if (el) {
                        const rect = el.getBoundingClientRect();
                        if (isHorizontal) {
                            indicatorRef.current.style.cssText = `left:${rect.left - tablistRect.left}px;width:${rect.width}px;opacity:1;`;
                        }
                        else {
                            indicatorRef.current.style.cssText = `top:${rect.top - tablistRect.top}px;opacity:1;`;
                        }
                    }
                }
            }
        }
    });
    useEffect(() => {
        updateIndicator();
    });
    useImperativeHandle(ref, () => ({
        updateIndicator,
    }), [updateIndicator]);
    return (_jsxs("div", Object.assign({}, restProps, { ref: tabsRef, className: getClassName(restProps.className, `${dPrefix}tabs`, `${dPrefix}tabs--${dPlacement}`, {
            [`${dPrefix}tabs--${dType}`]: dType,
            [`${dPrefix}tabs--${dSize}`]: dSize,
            [`${dPrefix}tabs--center`]: dCenter,
        }) }, { children: [_jsx("div", Object.assign({ ref: tablistWrapperRef, className: `${dPrefix}tabs__tablist-wrapper`, onScroll: () => {
                    refreshTabs();
                } }, { children: _jsxs("div", Object.assign({ ref: tablistRef, className: `${dPrefix}tabs__tablist`, role: "tablist", "aria-orientation": isHorizontal ? 'horizontal' : 'vertical' }, { children: [dList.map((item, index) => {
                            const { id: itemId, title: itemTitle, disabled: itemDisabled, closable: itemClosable } = item;
                            const getTab = (next, _index = index) => {
                                for (let focusIndex = next ? _index + 1 : _index - 1, n = 0; n < dList.length; next ? focusIndex++ : focusIndex--, n++) {
                                    const t = nth(dList, focusIndex % dList.length);
                                    if (t && !t.disabled) {
                                        return t;
                                    }
                                }
                            };
                            const focusTab = (t) => {
                                if (t) {
                                    changeActiveId(t.id);
                                    const el = document.getElementById(getTabId(t.id));
                                    if (el) {
                                        el.focus();
                                    }
                                }
                            };
                            const closeTab = () => {
                                if (activeId === itemId) {
                                    let hasTab = false;
                                    for (let focusIndex = index + 1; focusIndex < dList.length; focusIndex++) {
                                        const t = nth(dList, focusIndex);
                                        if (t && !t.disabled) {
                                            hasTab = true;
                                            focusTab(t);
                                            break;
                                        }
                                    }
                                    if (!hasTab) {
                                        for (let focusIndex = index - 1; focusIndex >= 0; focusIndex--) {
                                            const t = nth(dList, focusIndex);
                                            if (t && !t.disabled) {
                                                focusTab(t);
                                                break;
                                            }
                                        }
                                    }
                                }
                                onClose === null || onClose === void 0 ? void 0 : onClose(itemId, item);
                            };
                            const active = itemId === activeId;
                            return (_jsxs("div", Object.assign({ id: getTabId(itemId), className: getClassName(`${dPrefix}tabs__tab`, {
                                    'is-active': active,
                                    'is-disabled': itemDisabled,
                                    [`${dPrefix}tabs__tab--first`]: index === 0,
                                    [`${dPrefix}tabs__tab--last`]: index === dList.length - 1,
                                }), tabIndex: active && !itemDisabled ? 0 : -1, role: "tab", "aria-controls": getPanelId(itemId), "aria-selected": active, "aria-disabled": itemDisabled, onClick: () => {
                                    changeActiveId(itemId);
                                }, onKeyDown: (e) => {
                                    switch (e.code) {
                                        case 'Delete':
                                            e.preventDefault();
                                            if (itemClosable) {
                                                closeTab();
                                            }
                                            break;
                                        case 'ArrowLeft':
                                            e.preventDefault();
                                            if (dPlacement === 'top' || dPlacement === 'bottom') {
                                                focusTab(getTab(false));
                                            }
                                            break;
                                        case 'ArrowRight':
                                            e.preventDefault();
                                            if (dPlacement === 'top' || dPlacement === 'bottom') {
                                                focusTab(getTab(true));
                                            }
                                            break;
                                        case 'ArrowUp':
                                            e.preventDefault();
                                            if (dPlacement === 'left' || dPlacement === 'right') {
                                                focusTab(getTab(false));
                                            }
                                            break;
                                        case 'ArrowDown':
                                            e.preventDefault();
                                            if (dPlacement === 'left' || dPlacement === 'right') {
                                                focusTab(getTab(true));
                                            }
                                            break;
                                        case 'Home':
                                            e.preventDefault();
                                            for (const t of dList) {
                                                if (!t.disabled) {
                                                    focusTab(t);
                                                    break;
                                                }
                                            }
                                            break;
                                        case 'End':
                                            e.preventDefault();
                                            for (let index = dList.length - 1; index >= 0; index--) {
                                                if (!dList[index].disabled) {
                                                    focusTab(dList[index]);
                                                    break;
                                                }
                                            }
                                            break;
                                        default:
                                            break;
                                    }
                                } }, { children: [itemTitle, !itemDisabled && itemClosable && (_jsx("div", Object.assign({ className: `${dPrefix}tabs__close`, role: "button", "aria-label": t('Close'), onClick: (e) => {
                                            e.stopPropagation();
                                            closeTab();
                                        } }, { children: _jsx(CloseOutlined, { dSize: 14 }) })))] }), itemId));
                        }), (listOverflow || onAddClick) && (_jsxs("div", Object.assign({ className: `${dPrefix}tabs__button-container` }, { children: [listOverflow && (_jsx(DDropdown, Object.assign({ dList: dropdownList.map((tab) => {
                                        const { id: itemId, title: itemTitle, disabled: itemDisabled } = tab;
                                        return {
                                            id: itemId,
                                            label: (_jsx("span", Object.assign({ className: getClassName(`${dPrefix}tabs__dropdown-item`, {
                                                    'is-active': itemId === activeId,
                                                }) }, { children: itemTitle }))),
                                            type: 'item',
                                            disabled: itemDisabled,
                                        };
                                    }), dPlacement: dPlacement === 'left' ? 'bottom-left' : 'bottom-right', dCloseOnClick: false, onItemClick: (id) => {
                                        changeActiveId(id);
                                    } }, { children: _jsx("div", Object.assign({ className: getClassName(`${dPrefix}tabs__button`, `${dPrefix}tabs__button--more`, {
                                            'is-end': scrollEnd,
                                        }), style: {
                                            right: isHorizontal && onAddClick ? 52 : undefined,
                                            bottom: !isHorizontal && onAddClick ? 36 : undefined,
                                        }, "aria-label": t('More') }, { children: _jsx(EllipsisOutlined, { dSize: iconSize }) })) }))), onAddClick && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}tabs__button`, `${dPrefix}tabs__button--add`), role: "button", "aria-label": t('Add'), onClick: () => {
                                        onAddClick === null || onAddClick === void 0 ? void 0 : onAddClick();
                                    } }, { children: _jsx(PlusOutlined, { dSize: iconSize }) })))] }))), _jsx("div", { ref: indicatorRef, className: `${dPrefix}tabs__${dType ? dType + '-' : ''}indicator` })] })) })), dList.map((item) => {
                const { id: itemId, panel: itemPanel } = item;
                return (_jsx("div", Object.assign({ id: getPanelId(itemId), className: `${dPrefix}tabs__tabpanel`, tabIndex: 0, hidden: itemId !== activeId, role: "tabpanel", "aria-labelledby": getTabId(itemId) }, { children: itemPanel }), itemId));
            })] })));
}
export const DTabs = React.forwardRef(Tabs);
